@@ -6,13 +6,17 @@ require 'ostruct'
 
 options = OpenStruct.new
 # settin' some defaults
+options.systemd = 0
+options.sysvinit = 0
 options.systemd_el = 0
 options.systemd_sles = 0
+options.old_el = 0
+options.old_sles = 0
 options.sles = 0
 options.java = 'java-1.8.0-openjdk-headless'
-options.java_bin = '/usr/bin/java'
 options.release = 1
 options.platform_version = 0
+options.is_pe = false
 options.replaces = {}
 options.additional_dependencies = []
 options.user = 'puppet'
@@ -49,6 +53,9 @@ OptionParser.new do |opts|
   opts.on('--platform-version VERSION', Integer, 'VERSION of the puppet platform this builds for') do |v|
     options.platform_version = v
   end
+  opts.on('--[no-]enterprise-build', 'Whether or not this is a PE build') do |e|
+    options.is_pe = e
+  end
   opts.on('--replaces <PKG,VERSION>', Array, 'PKG and VERSION replaced by this package. Can be passed multiple times.') do |pkg,ver|
     options.replaces[pkg] = ver
   end
@@ -133,9 +140,17 @@ fail "--dist is required!" if options.output_type == 'deb' && options.dist.nil?
 if options.sources.empty?
   options.sources = case options.operating_system
                     when :amazon, :fedora, :sles, :el, :redhatfips
-                      ['etc', 'opt', 'usr', 'var']
+                      if options.operating_system == :el && options.os_version < 7 || options.operating_system == :sles && options.os_version < 12 #sysv rpm platforms
+                        ['etc', 'opt', 'var']
+                      else
+                        ['etc', 'opt', 'usr', 'var']
+                      end
                     when :debian, :ubuntu
-                      ['etc', 'lib', 'opt', 'var']
+                      if options.dist == 'trusty' #trusty is the only sysv deb we build for right now
+                        ['etc', 'opt', 'var']
+                      else
+                        ['etc', 'lib', 'opt', 'usr', 'var']
+                      end
                     else
                       fail "I don't know what your default sources should be, pass it on the command line!"
                     end
@@ -147,6 +162,7 @@ shared_opts = Array('')
 termini_opts = Array('')
 
 options.app_logdir = "/var/log/puppetlabs/#{options.realname}"
+options.app_rundir = "/var/run/puppetlabs/#{options.realname}"
 options.app_prefix = "/opt/puppetlabs/server/apps/#{options.realname}"
 options.app_data = "/opt/puppetlabs/server/data/#{options.realname}"
 
@@ -156,44 +172,68 @@ if options.output_type == 'rpm'
   shared_opts << "--rpm-digest sha256"
   shared_opts << "--rpm-rpmbuild-define 'rpmversion #{options.version}'"
   fpm_opts << "--rpm-rpmbuild-define '_app_logdir #{options.app_logdir}'"
+  fpm_opts << "--rpm-rpmbuild-define '_app_rundir #{options.app_rundir}'"
   fpm_opts << "--rpm-rpmbuild-define '_app_prefix #{options.app_prefix}'"
   fpm_opts << "--rpm-rpmbuild-define '_app_data #{options.app_data}'"
 
   if options.operating_system == :fedora # all supported fedoras are systemd
+    options.systemd = 1
     options.systemd_el = 1
   elsif options.operating_system == :amazon
     fpm_opts << "--depends tzdata-java"
     options.java = '(java-17-amazon-corretto-headless or java-11-amazon-corretto-headless)'
-
+    options.systemd = 1
     options.systemd_el = 1
   elsif options.operating_system == :el
     if options.os_version == 7
       options.java = 'jre-11-headless'
       options.java_bin = '/usr/lib/jvm/jre-11/bin/java'
-    elsif options.os_version >= 8
+    elsif (8..9).include?(options.os_version)
       options.java = 'jre-17-headless'
       options.java_bin = '/usr/lib/jvm/jre-17/bin/java'
+    elsif options.os_version == 10
+      options.java = 'jre-21-headless'
+      options.java_bin = '/usr/lib/jvm/jre-21/bin/java'
     else
       fail "Unrecognized el os version #{options.os_version}"
     end
 
+    options.systemd = 1
     options.systemd_el = 1
-  elsif options.operating_system == :redhatfips
+  elsif options.operating_system == :redhatfips && options.os_version >= 7 # systemd redhatfips
+    options.systemd = 1
     options.systemd_el = 1
-  elsif options.operating_system == :sles
+  elsif options.operating_system == :sles && options.os_version >= 12 # systemd sles
+    options.systemd = 1
     options.systemd_sles = 1
     options.sles = 1
-    options.java = 'java-11-openjdk-headless'
+    if ! options.is_pe
+      options.java = 'java-11-openjdk-headless'
+    end
+  elsif options.operating_system == :sles #old sles
+    options.sysvinit = 1
+    options.old_sles = 1
   end
 
+  fpm_opts << "--rpm-rpmbuild-define '_with_sysvinit #{options.sysvinit}'"
+  fpm_opts << "--rpm-rpmbuild-define '_with_systemd #{options.systemd}'"
+  fpm_opts << "--rpm-rpmbuild-define '_old_sles #{options.old_sles}'"
   fpm_opts << "--rpm-rpmbuild-define '_systemd_el #{options.systemd_el}'"
   fpm_opts << "--rpm-rpmbuild-define '_systemd_sles #{options.systemd_sles}'"
+  fpm_opts << "--rpm-rpmbuild-define '_old_el #{options.old_el}'"
   fpm_opts << "--rpm-rpmbuild-define '_sysconfdir /etc'"
   fpm_opts << "--rpm-rpmbuild-define '_prefix #{options.app_prefix}'"
+  fpm_opts << "--rpm-rpmbuild-define '_rundir /var/run'"
   fpm_opts << "--rpm-rpmbuild-define '__jar_repack 0'"
 
   shared_opts << "--rpm-dist #{options.dist}"
 
+  if options.old_el == 1
+    fpm_opts << "--depends chkconfig"
+  elsif options.old_sles == 1
+    fpm_opts << "--depends aaa_base"
+  end
+
   if options.systemd_el == 1
     fpm_opts << "--depends systemd"
   end
@@ -220,6 +260,7 @@ if options.output_type == 'rpm'
 
   fpm_opts << "--directories #{options.app_logdir}"
   fpm_opts << "--directories /etc/puppetlabs/#{options.realname}"
+  fpm_opts << "--directories #{options.app_rundir}"
   shared_opts << "--rpm-auto-add-directories"
   fpm_opts << "--rpm-auto-add-exclude-directories /etc/puppetlabs"
   shared_opts << "--rpm-auto-add-exclude-directories /opt/puppetlabs"
@@ -230,8 +271,13 @@ if options.output_type == 'rpm'
   fpm_opts << "--rpm-auto-add-exclude-directories /opt/puppetlabs/server/data"
   fpm_opts << "--rpm-auto-add-exclude-directories /usr/lib/systemd"
   fpm_opts << "--rpm-auto-add-exclude-directories /usr/lib/systemd/system"
+  fpm_opts << "--rpm-auto-add-exclude-directories /etc/init.d"
+  fpm_opts << "--rpm-auto-add-exclude-directories /etc/rc.d"
   fpm_opts << "--rpm-auto-add-exclude-directories /etc/logrotate.d"
+  fpm_opts << "--rpm-auto-add-exclude-directories /etc/rc.d/init.d"
+  fpm_opts << "--rpm-auto-add-exclude-directories /usr/lib/tmpfiles.d"
   fpm_opts << "--rpm-auto-add-exclude-directories /var/log/puppetlabs"
+  fpm_opts << "--rpm-auto-add-exclude-directories /var/run/puppetlabs"
   termini_opts << "--rpm-auto-add-exclude-directories /opt/puppetlabs/puppet"
   termini_opts << "--rpm-auto-add-exclude-directories /opt/puppetlabs/puppet/lib"
   termini_opts << "--rpm-auto-add-exclude-directories /opt/puppetlabs/puppet/lib/ruby"
@@ -251,6 +297,7 @@ if options.output_type == 'rpm'
   fpm_opts << "--rpm-attr 750,#{options.user},#{options.group}:/etc/puppetlabs/#{options.realname}"
   fpm_opts << "--rpm-attr 750,#{options.user},#{options.group}:#{options.app_logdir}"
   fpm_opts << "--rpm-attr -,#{options.user},#{options.group}:#{options.app_data}"
+  fpm_opts << "--rpm-attr 755,#{options.user},#{options.group}:#{options.app_rundir}"
 
   fpm_opts << "--edit"
   fpm_opts << "--category 'System Environment/Daemons'"
@@ -261,13 +308,19 @@ elsif options.output_type == 'deb'
     options.release = "#{options.release}+#{options.dist}"
   end
 
-  options.java = 'openjdk-17-jre-headless | openjdk-11-jre-headless'
+  if ! options.is_pe
+    options.java = 'openjdk-17-jre-headless | openjdk-11-jre-headless'
+  end
 
   fpm_opts << '--deb-build-depends cdbs'
   fpm_opts << '--deb-build-depends bc'
   fpm_opts << '--deb-build-depends mawk'
   fpm_opts << '--deb-build-depends lsb-release'
-  fpm_opts << '--deb-build-depends "ruby | ruby-interpreter"'
+  if options.is_pe
+    fpm_opts << '--deb-build-depends puppet-agent'
+  else
+    fpm_opts << '--deb-build-depends "ruby | ruby-interpreter"'
+  end
   fpm_opts << '--deb-priority optional'
   fpm_opts << '--category utils'
   options.deb_interest_triggers.each do |trigger|
@@ -330,9 +383,16 @@ if options.name == "openvoxdb"
   termini_opts << "--conflicts 'puppetdb-termini'"
 end
 
-fpm_opts << "--depends '#{options.java}'"
+if options.is_pe
+  fpm_opts << "--depends pe-java"
+  fpm_opts << "--depends pe-puppet-enterprise-release"
+  fpm_opts << "--depends pe-bouncy-castle-jars"
+else
+  fpm_opts << "--depends '#{options.java}'"
+end
 
 fpm_opts << "--depends bash"
+fpm_opts << "--depends net-tools"
 fpm_opts << "--depends /usr/bin/which" if options.output_type == 'rpm'
 fpm_opts << "--depends adduser" if options.output_type == 'deb'
 fpm_opts << "--depends procps"
